/**********************************************************/
/*                        T I A R E                       */
/* Astronomical Ressources & Ephemeris for TI Calculators */
/*               (c) 2003-2005, Yann DUCHEMIN             */
/* ------------------------------------------------------ */
/* This file is part of TIARE.                            */
/*                                                        */
/* TIARE is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public    */
/* License as published by the Free Software Foundation;  */
/* either version 2 of the License                        */
/*                                                        */
/* TIARE is distributed in the hope that it will be       */
/* useful, but WITHOUT ANY WARRANTY; without even the     */
/* implied warranty of MERCHANTABILITY or FITNESS FOR A   */
/* PARTICULAR PURPOSE. See the GNU General Public         */
/* License for more details.                              */
/*                                                        */
/* You should have received a copy of the GNU General     */
/* Public License along with TIARE; if not, write to      */
/* the Free Software Foundation, Inc., 59 Temple Place,   */
/* Suite 330, Boston, MA  02111-1307  USA                 */
/* ------------------------------------------------------ */
/* File Name : chronos.c                                  */
/* Created   : 27  II 2003                                */
/* Modified  : 31 III 2005                                */
/**********************************************************/

#include "chronos.h"

/* ====================================================== */
/* = Function    : Nb Days For a Month                  = */
/* = Input       : Short Month, Short year              = */
/* = Output      : number of day                        = */
/* ====================================================== */
short NbDaysFMonth(short month, short year)
{
 short int DMM[] = {31,28,31,30,31,30,31,31,30,31,30,31};
 // month = month - 1 because tabs start at zero ;o)
 if( (VerifyYearB(year)) )
  DMM[1] = 29;		// feb have 29 days
 return(DMM[month-1]);
}

/* ====================================================== */
/* = Function    : Year Validation                      = */
/* = Input       : Short year                           = */
/* = Output      : 1 if leap, if not 0                  = */
/* ====================================================== */
short VerifyYearB(short year)
{
 return (((year % 4 == 0) && ( (!(year % 100 == 0)) || (year % 400 == 0))) ? 1 : 0 );
}

/* ====================================================== */
/* = Function    : Day -> Decimal                       = */
/* = Input       : D,H,M,S			                        = */
/* = Output      : float decimal date DD.dd             = */
/* ====================================================== */
float Day2Float(short Day, short Hour, short Min, short Sec)
{
 float DD = 0.0;
 DD  = (float)Day;
 DD += (float)Hour/24.0;
 DD += (float)Min/1440.0;
 DD += (float)Sec/86400.0;
 return(DD); 
}

/* ====================================================== */
/* = Function    : Julian Day                           = */
/* = Input       : TIME structure,Flag if time at Oh00  = */
/* = Output      : float format Julian date             = */
/* ====================================================== */
float YMD2JD(struct TIME *ptime, short MidNight)
{
	float A = 0.0,	// A & B are correction for the Julian's calendar reform
				B = 0.0;	// In other terms we're always in the gredorian's calend
	unsigned short Y = ptime->ObsYear,
								 M = ptime->ObsMonth,
								 D = ptime->ObsDay;
	if( M < 3)
	{
		M +=12;
		Y -= 1;
	}
	A = Int_Float((float)Y/100.0);
	B = 2-A+Int_Float(A/4.0);
	B += Int_Float(365.25*(float)Y);				// C term
	B += Int_Float(30.6001*((float)M+1.0));		// D term
	if(!MidNight)
	B += (Day2Float(ptime->UTDay,ptime->UTHour,ptime->UTMin,ptime->UTSec)+1720994.5);			// A+B+C+D terms
	else
	B += ((float)D+1720994.5);
	return(B);	// the precision is only arround 0.1 sec
}

/* ====================================================== */
/* = Function    : convert DMS -> float                 = */
/* = Input       : Deg, Min, Sec values                 = */
/* = Output      : float                                = */
/* ====================================================== */
float DMS2Float(short deg, unsigned short min, unsigned short sec)
{
 float d = 0.0;
 d+=(float)fabs(deg);		// need a cast because a short was used
 d+=(float)min/60.0;
 d+=(float)sec/3600.0;
 (deg < 0.0) ? (d = d*(-1.0)) : d;
 return(d); 	
}

/* ====================================================== */
/* = Function    : DecimalDegrees to DMS                = */
/* = Input       : float degrees                        = */
/* = Output      : float struct D M S pointed by input  = */
/* ====================================================== */
void Float2DMS(float dec, struct DMS *dms)
{
 dms->d = Int_Float(dec); // 1 hour equ 15 degrees
 dms->m = Int_Float(fabs(dec-dms->d)*60.0);
 dms->s = ((fmod(fabs(dec-dms->d),60.0)*60.0)-dms->m)*60.0;
 if ( dms->s > 59.0 )
 {
 	dms->s = 0.0;
 	dms->m++;
 }
}

/* ====================================================== */
/* = Function    : DayOfWeek                            = */
/* = Input       : JDay AT Midnight for a given date    = */
/* = Output      : float between 0.0 to 6.0             = */
/* ====================================================== */
float DayOfWeek(float JD)
{
 return(fmod((JD+1.5),7.0));
}

/* ====================================================== */
/* = Function    : Day number                           = */
/* = Input       : TIME struct                          = */
/* = Output      : float                                = */
/* ====================================================== */
float DayNb(struct TIME *ptime)
{
 float n1, n2, k;
 n1 = Int_Float( (float)(ptime->ObsMonth*275)/9.0 );
 n2 = Int_Float( (float)(ptime->ObsMonth+9)/12.0 );
 k = 1.0 + Int_Float( (float)(ptime->ObsYear-4*Int_Float(ptime->ObsYear/4)+2) / 3.0 );
 return(n1-n2*k+ptime->ObsDay-30);
}

/* ====================================================== */
/* = Function    : Sideral Time at Greenwich in hours   = */
/* = Input       : float JD                             = */
/* = Output      : float SDT                            = */
/* ====================================================== */
float STG_H(float JD)
{
 float t = 0.0,
 			 sdt = 0.0;
 t = (JD-2451545.0)/36525.0;
 sdt = 100.46061837+t*(36000.770053608+t*(0.000387933-t/38710000.0));
 return(NormDeg(sdt)/15.0);
}

/* ====================================================== */
/* = Function    : Real Sideral Time at ObsLONGITUDE    = */
/* = Input       : float Sideral Tmoy and Real T struct = */
/* = Output      : float Real SDT                       = */
/* ====================================================== */
float RSTG_H(float STM, float ObsLONG, struct TIME *preal)
{
 float sdt = 0.0;
 sdt = STM + (DMS2Float((float)preal->UTHour,(float)preal->UTMin,(float)preal->UTSec)*1.00273790935);
 sdt -= ObsLONG/15.0;
 while(sdt < 0.0) sdt += 24.0;
 while(sdt > 24.0) sdt -= 24.0;
 return(sdt);		// this the real sideral time in hours
}

/* ====================================================== */
/* = Function    : LST for greenwich                    = */
/* = Input       : float T, TimeZone and longitude      = */
/* = Output      : float LST                            = */
/* ====================================================== */
float LST(float T, float Dtime, float Long)
{
	float s = 0.0;
	s += 24110.5+8640184.813*(T/36525.0);
	s += (86636.6*(Dtime/24.0) + (86400.0*Long))/86400.0;
	s -= Int_Float(s);
	return(s*Deg2Rad(360.0));	
}

/* ====================================================== */
/* = Function    : DST                                  = */
/* = Input       : float DATE structure                 = */
/* = Output      : short 0 or 1 if Daylight Save Time   = */
/* ====================================================== */
short DST(struct TIME *ptime)
{
 struct TIME dst;
 struct TIME *pdst;
 pdst = &dst;
 
 // initialize structure to search
 pdst->ObsDay = 31;
 pdst->ObsMonth = 3;
 pdst->ObsYear = ptime->ObsYear;
 pdst->ObsHour = ptime->ObsHour;
 pdst->ObsMin = ptime->ObsMin;
 pdst->ObsSec = ptime->ObsSec;
 
 float  dst_start = 0.0,
 				dst_end = 0.0;
 				
 while(DayOfWeek(dst_start = YMD2JD(pdst,MIDNIGHT)) != 0.0)
 	pdst->ObsDay--;	// for testing day before the last Sunday's month

 pdst->ObsDay = 31;
 pdst->ObsMonth = 10;

 while(DayOfWeek(dst_end = YMD2JD(pdst,MIDNIGHT)) != 0.0)
 	(short)pdst->ObsDay--;	// for testing day before the last Sunday's month
 
 if(YMD2JD(ptime,REALTIME) > dst_start &&	YMD2JD(ptime,REALTIME) < dst_end)
 	return(1);
 else
  return(0);
}

/* ====================================================== */
/* = Function    : UTC                                  = */
/* = Input       : struct TIME and Time Zone            = */
/* = Output      : nothing                              = */
/* ====================================================== */
void UTC(struct TIME *ptime, short DTime)
{
 short dec = abs(DTime);

// the net adjust the time correction
 if( !(DST(ptime)) ) 
 {
  if( !(DTime >= 0) )
 	{
 	 (short)ptime->UTHour+=dec; 	 
  }
  else
  {
   (short)ptime->UTHour-=dec;
  } 
 }
 else
 {
	if( !(DTime >= 0) )
	{
 	 (short)ptime->UTHour+=(dec-1);
	}
	else
 	{
	 (short)ptime->UTHour-=(dec+1);
 	}
 }

// date's control validity 
 	 if( (short)ptime->UTHour < 0 )	// time must be positive
 	 {
  	(short)ptime->UTDay -= 1;			// else we're a day before
 	 	ptime->UTHour = 24+fabs((float)ptime->UTHour);

 	  if( ptime->UTDay <= 1 )			// test for the month before
 	  {
		 (short)ptime->UTMonth-=1;
		 ptime->UTDay = NbDaysFMonth(ptime->UTMonth, ptime->UTYear);
  	 if( ptime->UTMonth <= 1 )
  	 {
      ptime->UTYear -= 1;
      ptime->UTMonth = 12;
	 	  ptime->UTDay = 31;
 	   }
 	  }
 	 }
 	 
 	 if( (short)ptime->UTHour >= 24 )
 	 {
  	(short)ptime->UTDay += 1;
  	if( !DST(ptime) )
  	{
  	 	ptime->UTHour -= 24;
  	}
  	else
  	{
  	 (short)ptime->UTHour -= 23; 
  	}
 	  if( (short)ptime->UTDay > NbDaysFMonth(ptime->UTMonth, ptime->UTYear) )
 	  {
		 (short)ptime->UTMonth+=1;
		 ptime->UTDay = 1;
  	 if( ptime->UTMonth > 12 )
  	 {
      ptime->UTYear += 1;
      ptime->UTMonth = 1;
	 	  ptime->UTDay = 1;
 	   }
 	  }
 	 }
//end of control 
}

/* ====================================================== */
/* = Function    : Time Scale at 12h00 UT               = */
/* = Input       : struct TIME                          = */
/* = Output      : float                                = */
/* ====================================================== */
float TScl(struct TIME *ptime)
{
 float m, c, l, r;
 m = 357.0 + 0.9856 * DayNb(ptime);
 c = 1.914 * sinD(m) + 0.02 * sinD(2.0*m);
 l = 280.0 + c + 0.9856 * DayNb(ptime);
 r = -2.466 * sinD(2.0*l) + 0.053 * sinD(4.0*l);
 return((c+r)*4.0);
}

/* ====================================================== */
/* = Function    : Kepler's Equation                    = */
/* = Input       : mean anomaly, Eccentricity, precision= */
/* = Output      : float                                = */
/* ====================================================== */
float KeplerEq(float m, float e, float p)
{
 float e0 = m,
 			 e1 = m;
 do
 {
  e0 = e1;
  e1 = e0+( ((m+Rad2Deg(e)*sinD(e0))-e0) / (1-e*cosD(e0)) );
 } while( ((e1-e0) > p) );
 return(e1);
}

/* ====================================================== */
/* = Function    : Compute Rise & Set for an object     = */
/* = Input       : Time, Place, Objet, Type             = */
/* = Output      : nothing                              = */
/* ====================================================== */
void RiseAndSet(struct TIME *ptime, struct PLACE *pplace, struct OBJECT *pobj, short obj)
{
 unsigned short uth = ptime->UTHour,
 							  utm = ptime->UTMin,
 							  uts = ptime->UTSec;

 ptime->UTHour  = 12;
 ptime->UTMin   = 0;
 ptime->UTSec   = 0;
 
 float h = -0.00989;	// Diameter under Horizon for small object
 float hr= 15.04107;	// rotation angle per hour
// float LONGobs = DMS2Float(pplace->LongDeg, pplace->LongMin, pplace->LongSec);
 float LATobs = DMS2Float(pplace->LatDeg, pplace->LatMin, pplace->LatSec);

 float UT_RS = 0.0,		// object at middle day
 		   CLHA = 0.0;		// COSINUS LOCAL ANGLE

  if( obj == SUN )
  {
  	 h = -0.01454;
  	 hr = 15.0;  	
  }
  else if( obj == MOON)
  {
  	h = 0.00217;
  }
 		   
  UT_RS = 12.0 + pobj->RA - RSTG_H(STG_H(YMD2JD(ptime, REALTIME)), DMS2Float(pplace->LongDeg, pplace->LongMin, pplace->LongSec), ptime); 
  if(UT_RS < 0.0)
  {
   UT_RS += 24.0 ;
  }
  else if(UT_RS > 24.0)
  {
   UT_RS -= 24.0;
  }

  CLHA = ((h-sinD(LATobs)*sinD(pobj->Dec)));
  CLHA /= ((cosD(LATobs)*cosD(pobj->Dec)));
  
  if(!((CLHA <= -1.0) && (CLHA >= 1.0)))
  {
  	CLHA = Rad2Deg(acos(CLHA))/hr;
  	pobj->rise = (UT_RS - CLHA < 0.0) ? 24.0 + UT_RS - CLHA  : UT_RS - CLHA ;
  	pobj->set = (UT_RS + CLHA > 24.0) ? UT_RS + CLHA - 24.0  : UT_RS + CLHA ;
  }
  else	// Not rise or not set
  {
  	pobj->rise = 0.0;
  	pobj->set = 0.0;
  }
 ptime->UTHour  = uth;
 ptime->UTMin   = utm;
 ptime->UTSec   = uts;
}