/**********************************************************/
/*                        T I A R E                       */
/* Astronomical Ressources & Ephemeris for TI Calculators */
/*               (c) 2003-2005, Yann DUCHEMIN             */
/* ------------------------------------------------------ */
/* This file is part of TIARE.                            */
/*                                                        */
/* TIARE is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public    */
/* License as published by the Free Software Foundation;  */
/* either version 2 of the License                        */
/*                                                        */
/* TIARE is distributed in the hope that it will be       */
/* useful, but WITHOUT ANY WARRANTY; without even the     */
/* implied warranty of MERCHANTABILITY or FITNESS FOR A   */
/* PARTICULAR PURPOSE. See the GNU General Public         */
/* License for more details.                              */
/*                                                        */
/* You should have received a copy of the GNU General     */
/* Public License along with TIARE; if not, write to      */
/* the Free Software Foundation, Inc., 59 Temple Place,   */
/* Suite 330, Boston, MA  02111-1307  USA                 */
/* ------------------------------------------------------ */
/* File Name : tiare_.c                                   */
/* Created   : 27 II  2003                                */
/* Modified  : 06 V   2005                                */
/**********************************************************/

#include "tiare_.h"

/* ====================================================== */
/* = Function    : Extract the integer part of float    = */
/* = Input       : float                                = */
/* = Output      : float                                = */
/* ====================================================== */
float Int_Float(float number)
{
 float v = 0.0;
 modf(number, &v);
 return(v);
}

/* ====================================================== */
/* = Function   : convert degrees -> radix              = */
/* ====================================================== */
inline float Deg2Rad( float alpha )
{
 return( (alpha*PI) / 180.0 );
}

/* ====================================================== */
/* = Function   : convert radix -> degrees              = */
/* ====================================================== */
inline float Rad2Deg( float alpha )
{
 return( (alpha*180.0) / PI );
}

/* ====================================================== */
/* = Function    : Normalise expression between 0 & 1   = */
/* = Input       : float                                = */
/* = Output      : float                                = */
/* ====================================================== */
float NormExp(float val)
{
 val -= Int_Float(val);
 if(val < 0.0)
  val++;
 return(val);
}

/* ====================================================== */
/* = Function    : Normalise Degrees between 0 & 360    = */
/* = Input       : float, normalized angle between TRIG = */
/* = Output      : float                                = */
/* ====================================================== */
float NormDeg(float val)
{
 val /= 360.0;
 val -= Int_Float(val);
 val *= 360.0;
//  if(val < 0.0)
//   val+=360;
 return(val);
}

/* ====================================================== */
/* = Function    : Normalise Degrees between 0 & 180    = */
/* = Input       : float                                = */
/* = Output      : float                                = */
/* ====================================================== */
float Norm180(float val)
{
 val /= 180.0;
 val -= Int_Float(val);
 val *= 180.0;
 return(val);
}

/* ====================================================== */
/* = Function    : Degrees' SIN 										    = */
/* = Input       : float                                = */
/* = Output      : float                                = */
/* ====================================================== */
float sinD(float val)
{
 return(sin(Deg2Rad(val)));
}

/* ====================================================== */
/* = Function    : Degrees' COS 										    = */
/* = Input       : float                                = */
/* = Output      : float                                = */
/* ====================================================== */
float cosD(float val)
{
 return(cos(Deg2Rad(val)));
}

/* ====================================================== */
/* = Function    : Calculate the phase of planet        = */
/* = Input       : SUN's Coords, object coords	        = */
/* = Output      : angle's phase in degrees             = */
/* ====================================================== */
float PlanetPhase(float RA_SUN, float DEC_SUN, float RA, float DEC)
{
 float Dif = 0.0;	
 float angle = 0.0;	
 	
 RA_SUN *= 15.0;
 RA *= 15.0;

 Dif = RA_SUN-RA; // position angle illuminated
 angle = Norm180(cosD(DEC_SUN)*sinD(Dif));
 angle = atan2(angle,Norm180((cosD(DEC)*sinD(DEC_SUN)-sinD(DEC)*cosD(DEC_SUN)*cosD(Dif)))); 
 return(Rad2Deg(angle));
}

/* ====================================================== */
/* = Function    : Compute position's object				    = */
/* = Input       : origin. days, struct Object, object  = */
/* = Output      : -                                    = */
/* ====================================================== */
void PosObj(float LST, struct TIME *ptime, struct PLACE *pplace, struct OBJECT *pobj, short obj)
{
 float n = 0.0,			// N = longitude of the ascending node -> usually OMEGA
 			 i = 0.0,			// i = inclination to the ecliptic (plane of the Earth's orbit)
 			 w = 0.0,			// w = argument of perihelion
 			 a = 0.0,			// a = semi-major axis, or mean distance from Sun
 			 e = 0.0,			// e = eccentricity (0=circle, 0-1=ellipse, 1=parabola)
 			 Ln = 0.0,
 			 obl= 0.0,		// obliquity
 			 ee = 0.0,
 			 m0 = 0.0,		// M = mean anomaly (0 at perihelion; increases uniformly with time)
 			 m1 = 1.0,
 			 v = 0.0,
 			 yec = 0.0,
 			 zec = 0.0,
 			 d = 0.0;
 			 
 float w_sun = 0.0,
 			 e_sun = 0.0,
 			 m_sun = 0.0,
 			 ee_sun = 0.0,
 			 x_sun = 0.0,
 			 y_sun = 0.0,
 			 r_sun = 0.0,
 			 v_sun = 0.0,
 			 long_sun = 0.0,
 			 ra_sun = 0.0,
 			 dec_sun = 0.0;

 float LATobs = DMS2Float(pplace->LatDeg, pplace->LatMin, pplace->LatSec);

 d = (YMD2JD(ptime, REALTIME)-2451543.5); // Nb of days since 2000.0 Ephem references

 // compute SUN's parameters
 w_sun = (282.9404+0.0000470935 * d);
 obl = (23.4393-0.0000003563 * d);
 e_sun = (0.016709 - 0.000000001151 * d);
 m_sun = (356.0470+0.9856002585 * d );
 ee_sun = m_sun+180/PI*e_sun*sinD(m_sun)*(1+e_sun*cosD(m_sun));
 x_sun = cosD(ee_sun)-e_sun;
 y_sun = sinD(ee_sun)*sqrt(1-e_sun*e_sun);
 r_sun = sqrt(x_sun*x_sun + y_sun*y_sun);
 v_sun = Rad2Deg(atan2(Deg2Rad(y_sun),Deg2Rad(x_sun)));
 long_sun = NormDeg( w_sun+v_sun);
 x_sun = r_sun*cosD(long_sun);
 y_sun = r_sun*sinD(long_sun);
 ra_sun = NormDeg(Rad2Deg(atan2(Deg2Rad(y_sun*cosD(obl)),Deg2Rad(x_sun)))) / 15.0;
 (ra_sun > 0.0) ? : (ra_sun += 24.0);
 dec_sun = NormDeg(Rad2Deg(asin(y_sun*sinD(obl)/r_sun)));
 /* Note: Orbitals elements can be found on the JPL website */
 /* http://ssd.jpl.nasa.gov/elem_planets.html */
 switch (obj)
 {
 	case SUN: 
 	pobj->LONG = long_sun;
 	pobj->DE = r_sun;
 	pobj->DS = 0.0;
	pobj->RA = ra_sun;
 	pobj->Dec = dec_sun;
  LST -= ra_sun;
  LST *= 15.0;
  pobj->xa = cosD(LST)*cosD(pobj->Dec);
  pobj->ya = sinD(LST)*cosD(pobj->Dec);
  pobj->za = sinD(pobj->Dec);
  pobj->xe = pobj->xa*sinD(LATobs)-pobj->za*cosD(LATobs);
  pobj->ze = pobj->xa*cosD(LATobs)+pobj->za*sinD(LATobs);
  // compute azimuth
  pobj->Az = NormDeg(Rad2Deg(atan2(Deg2Rad(pobj->ya),Deg2Rad(pobj->xe)))+180);
  // compute altitude above the horizon's observer
  pobj->Al = NormDeg(Rad2Deg(asin(pobj->ze)));
//  pobj->Dec = NormDeg(Rad2Deg(atan2(Deg2Rad(pobj->ze),Deg2Rad(sqrt((pobj->xe*pobj->xe)+(pobj->ye*pobj->ye))))));
  pobj->Ph = 90.0;
  // compute rise & set times
  RiseAndSet(ptime, pplace, pobj, SUN);
  return; // exit from this function
 	break;
 	case MERCURY:
  	n = (48.33167 + 0.0000324587 * d);	    	 // Long of asc. node
    i = (7.00487 + 0.0000000500 * d);					 // Inclination
    w = (29.1241 + 0.0000101444 * d);					 // Argument of perihelion
    a = 0.38709893;                            // Semi-major axis
    e = (0.20563069 + 0.000000000559 * d);		 // Eccentricity
    m0 = (168.6562 + 4.0923344368 * d);				 // Mean anonaly
		m1 = 102.27938 + 149472.51529*d + 0.000007*(d*d);
 	break;
 	case MARS:
   n  = (49.57854 + 0.000021081 * d);
   i  = (1.85061 + 0.0000000178 * d);
   w  = (286.5016 + 0.0000292961 * d);
   e  = (0.093405 + 0.000000002516 * d);
   m0 = (18.6021 + 0.5240207766 * d);
   m1 = 319.51913 + 19139.85475*d + 0.000181*(d*d);
   a  = 1.52366231;
  break;
 	case VENUS:
   n  = (76.68069 + 0.0000246590 * d);
   i  = (3.39471 + 0.0000000275 * d);
   w  = (54.8910 + 0.0000138374 * d);
   e  = (0.006773 - 0.000000001302 * d);
   m0 = (48.0052 + 1.60213002244 * d);
   m1 = 212.60322 + 58517.80387*d + 0.001286*(d*d);
   a  = 0.72333199;
  break;
 	case SATURN:
   n  = (113.71504 + 0.0000238980 * d);
   i  = (2.48446 - 0.0000001081 * d);
   w  = (339.3939 + 0.0000297661 * d);
   e  = (0.055546 - 0.000000009499 * d);
   m0 = (316.9670 + 0.0334442282 * d);
   m1 = 175.46622 + 1221.55147*d + 0.000502*(d*d);
   a  = 9.53707032;
   Ln = (0.812*sinD(2*m0-5*m1-67.6)-0.229*cosD(2*m0-4*m1-2)+0.119*sinD(m0-2*m1-3)+0.046*sinD(2*m0-6*m1-69)+0.014*sinD(m0-3*m1+32));
  break;
  case JUPITER:
   n  = (100.55615 + 0.0000276854 * d);
   i  = (1.30530 + 0.0000001557 * d);
   w  = (273.8777 + 0.0000164505 * d);
   e  = (0.048498 + 0.000000004469 * d);
   m0 = (19.8950 + 0.0830853001 * d);
   m1 = 225.32833 + 3034.69202*d + 0.000722*(d*d);
   a  = 5.20336301;
   Ln = -(0.332*sinD(2*m0-5*m1-67.6)-0.056*sinD(2*m0-2*m1+21)+0.042*sinD(3*m0-5*m1+21)-0.036*sinD(m0-2*m1)+0.022*cosD(m0-m1)+0.023*sinD(2*m0-3*m1+52)-0.016*sinD(m0-5*m1-69));
  break;
  case MOON:
   n  = (125.1228 - 0.0529538083 * d);
   i  = 5.1454;
   w  = (318.0634 + 0.1643573223 * d);
   e  = 0.054900;
   m0 = (115.3654 + 13.0649929509 * d);
   m1 = NormDeg(n+w+m0);
   a  = 60.2666;
   Ln = (-1.274*sinD(m0-2*m1) + 0.658*sinD(2*m1) - 0.186*sinD(m_sun) - 0.059*sinD(2*m0-2*m1) - 0.057*sinD(m0-2*m1+m_sun) + 0.053*sinD(m0+2*m1) + 0.046*sinD(2*m1-m_sun) + 0.041*sinD(m0-m_sun) - 0.035*sinD(m1) - 0.031*sinD(m0+m_sun) - 0.015*sinD(2*(m1-n)-2*m1) + 0.011*sinD(m0-4*m1));
  break;
 }
 
 pobj->Lm = NormDeg(w + m0) + 180;				// medium LONG
 ee = KeplerEq(m0,e,KeplerP);
 pobj->xa = (a*(cosD(ee)-e));
 pobj->ya = (a*sqrt(1.0-e*e)*sinD(ee));
 v = Rad2Deg(atan2(Deg2Rad(pobj->ya),Deg2Rad(pobj->xa)));
 pobj->DS = sqrt(pobj->xa*pobj->xa+pobj->ya*pobj->ya); // mean r
 pobj->xe = pobj->DS*(cosD(n)*cosD(v+w)-sinD(n)*sinD(v+w)*cosD(i));
 pobj->ye = pobj->DS*(sinD(n)*cosD(v+w)+cosD(n)*sinD(v+w)*cosD(i));
 pobj->ze = pobj->DS*(sinD(v+w)*sinD(i));
 pobj->LONG = NormDeg(Rad2Deg(atan2(Deg2Rad(pobj->ye),Deg2Rad(pobj->xe))) + Ln);
 pobj->LAT  = NormDeg(Rad2Deg(asin(pobj->ze/pobj->DS)));

 if( obj == MOON) 
 {
 	pobj->LAT = pobj->LAT - 0.173*sinD((m1-n)-2*m1) - 0.055*sinD(m0-(m1-n)-2*m1) - 0.046*sinD(m0+(m1-n)-2*m1) + 0.033*sinD((m1-n)+2*m1) + 0.017*sinD(2*m0+(m1-n)); //perturbations in LATITUDE
 }
 
 //--- search for equatorials coords.
 pobj->xe = pobj->DS*cosD(pobj->LONG)*cosD(pobj->LAT);
 yec = pobj->DS*sinD(pobj->LONG)*cosD(pobj->LAT);
 zec = pobj->DS*sinD(pobj->LAT);
 if( m_sun != 1)
 {
 	pobj->xe = pobj->xe + x_sun;
 	yec += y_sun;
 }
 pobj->ye = yec*cosD(obl)-zec*sinD(obl);
 pobj->ze = yec*sinD(obl)+zec*cosD(obl);
 // Compute the Right Ascension
 // It's necessary to divide the result by 15 to convert degrees in hours
 pobj->RA = NormDeg(Rad2Deg(atan2(Deg2Rad(pobj->ye),Deg2Rad(pobj->xe))))/15.0;
 (pobj->RA > 0.0) ? : (pobj->RA = pobj->RA+24.0);
 // Compute the Declinaison
 pobj->Dec= NormDeg(Rad2Deg(atan2(Deg2Rad(pobj->ze),Deg2Rad(sqrt(pobj->xe*pobj->xe+pobj->ye*pobj->ye)))));
 pobj->DE = sqrt(pobj->xe*pobj->xe+pobj->ye*pobj->ye+pobj->ze*pobj->ze); // mean re
 
 if( obj == MOON )
 {
	pobj->DE = pobj->DE - 0.58*cosD(m0-2*m1) - 0.46*cosD(2*m1);	// perturbations in lunar distance
 }

 //--- search for azimutals coords.
 LST -= pobj->RA;
 LST *= 15.0;
 pobj->xa = cosD(LST)*cosD(pobj->Dec);
 pobj->ya = sinD(LST)*cosD(pobj->Dec);
 pobj->za = sinD(pobj->Dec);
 pobj->xe = pobj->xa*sinD(LATobs)-pobj->za*cosD(LATobs);
 pobj->ze = pobj->xa*cosD(LATobs)+pobj->za*sinD(LATobs);
 // compute azimuth
 pobj->Az = NormDeg(Rad2Deg(atan2(Deg2Rad(pobj->ya),Deg2Rad(pobj->xe))+PI));
 // compute altitude above the horizon's observer
 pobj->Al = NormDeg(Rad2Deg(asin(pobj->ze)));
 // computing object phase with Meuus algorithm applicated to the moon illumination angle
 pobj->Ph = PlanetPhase(ra_sun, dec_sun, pobj->RA, pobj->Dec);
 // compute rise & set times
 RiseAndSet(ptime, pplace, pobj, 99);
}

/* ====================================================== */
/* = Function   : MAIN                                  = */
/* ====================================================== */
void _main(void)
{
 // program's variables
 struct PLACE LocalObs;	// Declare observer's position
 struct PLACE *pplace;	// pointer for LocalObs
 pplace = &LocalObs;		// initialise pointer pplace to the adress of the LocalObs' first element

 struct TIME TimeObs;		// Declare default time observation
 struct TIME *ptime;		// pointer for TimeObs
 ptime = &TimeObs;			// initialise pointer time to the adress of the TimeObs' first element

 struct PARAM ExtParam;	// Declare default extras parameters
 struct PARAM *pparam;	// pointer for ExtParam
 pparam = &ExtParam;		// initialise pointer pparam to the adress of the ExtParam's first element

 //verify if a default configuration exist
 if( RFchTime(ptime) || RFchLocal(pplace) || RFchParam(pparam) ) 
 {
 	DlgMessage ("Warning", "Missing Conf. !", BT_NONE, BT_OK);
 }
 //start apps
 MainMenu(ptime, pplace, pparam);
}
