/**********************************************************/
/*                        T I A R E                       */
/* Astronomical Ressources & Ephemeris for TI Calculators */
/*               (c) 2003-2005, Yann DUCHEMIN             */
/* ------------------------------------------------------ */
/* This file is part of TIARE.                            */
/*                                                        */
/* TIARE is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public    */
/* License as published by the Free Software Foundation;  */
/* either version 2 of the License                        */
/*                                                        */
/* TIARE is distributed in the hope that it will be       */
/* useful, but WITHOUT ANY WARRANTY; without even the     */
/* implied warranty of MERCHANTABILITY or FITNESS FOR A   */
/* PARTICULAR PURPOSE. See the GNU General Public         */
/* License for more details.                              */
/*                                                        */
/* You should have received a copy of the GNU General     */
/* Public License along with TIARE; if not, write to      */
/* the Free Software Foundation, Inc., 59 Temple Place,   */
/* Suite 330, Boston, MA  02111-1307  USA                 */
/* ------------------------------------------------------ */
/* File Name : tiare.h                                    */
/* Created   : 27 II  2003                                */
/* Modified  : 21 VII 2005                                */
/**********************************************************/

/* ====================================================== */
/* = Compilator's Directives                            = */
/* ====================================================== */
#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200		          // Compile for V200

// Rom Call opt. wasn't recommended with floating point calculations
// #define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define MIN_AMS 200           // Compile for AMS 2.00 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#define KERNEL_FORMAT_ROM_CALLS
#define EXECUTE_IN_GHOST_SPACE
#define OPTIMIZE_CALC_CONSTS
#define SET_FILE_IN_USE_BIT

// gcc options to use :
// -Os -Wall -W -Wwrite-strings -ffunction-sections -fdata-sections
// a68k:  -g -t

/* ====================================================== */
/* = Poject Definitions                                 = */
/* ====================================================== */
#include <compat.h>
#include <graph.h>
#include <stdio.h>
#include <stdlib.h>
#include <dialogs.h>
#include <args.h>
#include <string.h>
#include <kbd.h>
#include <math.h>
#include <alloc.h>

/* ====================================================== */
/* = Common Definitions                                 = */
/* ====================================================== */
#define		VERSION		"0.06"

#define		REALTIME	0
#define		MIDNIGHT	1

#define		TRIG_OFF	0
#define		TRIG_ON		1

#define		ZOOM_MAX	5

#define		PLANET		1
#define		STAR			2
#define		_1AU			149597870			// One astronomical unit in kilometers
#define		KeplerP		0.00000000001	// precision for Kepler's equation

#define		SUN				0
#define		MERCURY		1
#define		VENUS			2
#define		EARTH			3
#define		MOON			31
#define		MARS			4
#define		JUPITER		5
#define		SATURN		6

#define		EQU_M			1							// equatorial mount
#define		ALT_M			2							// altazimutal mount
#define		GER_M			3							// german mount

#define		NOT_RS		0							// Mount not connected
#define		LX_200		1							// mount uses LX200 protocol
#define		ASCOM			2							// mount uses ASCOM protocol

#define SUNDIAMSEC	1919.26/60.0	// Sun's diameter in arcseconds seen from 1AU
#define MOONDIAMSEC	1873.7				// Moon's diameter in earth radii
#define MOONDIAMIN	6378.14
#define MeDIAMSEC		6.74				  // Mercury's diameter in arcseconds seen from 1AU
#define JDIAMSEC		196.94				// Jupiter's diameter in arcseconds seen from 1AU
#define SDIAMSEC		165.6					// Saturn's diameter in arcseconds seen from 1AU
#define MDIAMSEC		9.36					// Mars's diameter in arcseconds seen from 1AU
#define VDIAMSEC		16.92					// Venus's diameter in arcseconds seen from 1AU

#define	MSG_INFO	0
#define	MSG_ERR		1
#define	MSG_FILE	2
#define	MSG_NIMP	3
#define	MSG_MEM		4
#define	MSG_END		5
#define MSG_CNT		6
#define MSG_KEY		7
#define MSG_KEY0	8
#define MSG_OBS		9
#define MSG_SEC		10
#define MSG_MIN		11
#define MSG_HRS		12
#define MSG_DAY		13
#define MSG_MTH		14
#define MSG_YER		15
#define END_PRG		99

// create a structure
// to define the observers' coords
// this one doesn't keep any memory because no
// var name is uses
struct PLACE
{
	 short int LatDeg;						// Latitude Degrees
	 unsigned short int LatMin;		// Latitude Minutes
	 unsigned short int LatSec;		// Latitude Secondes
	 short int LongDeg;						// Longitude Degree
	 unsigned short int LongMin;	// Longitude Minutes
	 unsigned short int LongSec;	// Longitude Secondes
	 short int Alt;
	 short int DTime;		// Est(+) & West(-) [TimeZone]
};

// create a struture to save the date & time
// this one doesn't keep any memory because no
// var name is uses
struct TIME
{
	unsigned short int ObsDay;		// Observation Day
	unsigned short int ObsMonth;	//	Month
	unsigned short int ObsYear;		//	Year
	unsigned short int ObsHour;		//  Hour
	unsigned short int ObsMin;		//  Min.
	unsigned short int ObsSec;		//  Sec.
	unsigned short int UTDay;		//  Universal Date
	unsigned short int UTMonth;
	unsigned short int UTYear;
	unsigned short int UTHour;
	unsigned short int UTMin;
	unsigned short int UTSec;
};

// create a struture to save extras parameters
// this one doesn't keep any memory because no
// var name is uses
struct PARAM
{
	unsigned short int GRS_Ref;		// Jupiters' red spot reference in degrees
	unsigned short int Zoom;			// Zoom factor for graphical representation
	short int PullDownBuffer[3];	// Tags for mount type & protocol
};

// create a structure to save the object relative data
// this one doesn't keep any memory because no
// var name is uses
struct OBJECT
{
	float LONG;				// LONGitude
	float LAT;				// latitude
	float	xa;					// azimut coords.
	float ya;
	float za;
	float	xe;					// Equ. coords.
	float ye;
	float ze;
	float DS;					// distance from the sun
	float DE;					// distance from the earth
	float RA;					// right ascension
	float Dec;				// declinaison
	float Az;					// azimuth
	float Al;					// altitude
	float Lm;					// medium LONG
	float Ph;					// phase seen from earth
	float rise;
	float set;
};

// create a structure to convert format DMS
// this one doesn't keep any memory because no
// var name is uses
struct DMS
{
	float d;
	float m;
	float s;
};

/* ====================================================== */
/* = tiare.c PROTOTYPES                                 = */
/* ====================================================== */
void _main(void);
inline float Deg2Rad(float);
inline float Rad2Deg(float);
float Int_Float(float);
float NormExp(float);
float NormDeg(float);
float Norm180(float);
float sinD(float);
float cosD(float);
float PlanetPhase(float, float, float, float);
void PosObj(float, struct TIME *, struct PLACE *, struct OBJECT *, short);

/* ====================================================== */
/* = pict.c PROTOTYPES                                  = */
/* ====================================================== */
void DrawScrBorder(void);
void DrawCopyright(void);
void DrawCircle(unsigned short, unsigned short, unsigned int);
short DrawStar(unsigned short, unsigned short, unsigned short);
short DrawingPosObj(short, struct TIME *, struct PLACE *, struct PARAM *);
short DrawCalendar(struct TIME *);
short DrawShadow(unsigned short, unsigned short, unsigned short);
void DrawSphere(unsigned short, unsigned short, unsigned short, float, float, float);
void DrawSky(void);

/* ====================================================== */
/* = moon.c PROTOTYPES                                  = */
/* ====================================================== */
short DrawMoon(struct TIME*, struct PLACE *);

/* ====================================================== */
/* = sun.c PROTOTYPES                                   = */
/* ====================================================== */
// short SunEffects(struct TIME *, struct PLACE *);

/* ====================================================== */
/* = menu.c definitions                                 = */
/* ====================================================== */
short MainMenu(struct TIME *, struct PLACE *, struct PARAM *);
short EdTime(struct TIME *, short);
short EdPlace(struct PLACE *);
short EdParam(struct PARAM *);

/* ====================================================== */
/* = chronos.c PROTOTYPES                               = */
/* ====================================================== */
short NbDaysFMonth(short, short);
short VerifyYearB(short);
float YMD2JD(struct TIME *, short);
float Day2Float(short, short, short, short);
float DayNb(struct TIME *);
float DMS2Float(short , unsigned short , unsigned short);
void  Float2DMS(float, struct DMS *);
float DayOfWeek(float);
float LST(float ,float, float);
float STG_H(float);
float RSTG_H(float, float, struct TIME *);
short DST(struct TIME *);
void  UTC(struct TIME *, short);
float TScl(struct TIME *);
float KeplerEq(float, float, float);
void RiseAndSet(struct TIME *, struct PLACE *, struct OBJECT *, short);

/* ====================================================== */
/* = fich.c PROTOTYPES                                  = */
/* ====================================================== */
short RFchTime(struct TIME *);
short RFchLocal(struct PLACE *);
short RFchParam(struct PARAM *);
short FchTime(struct TIME *, short);
short FchLocal(struct PLACE *);
short FchParam(struct PARAM *);

/* ====================================================== */
/* = errors.c PROTOTYPES                                = */
/* ====================================================== */
void  ErrMsg(int, int);
short VerifyTimeStruct(struct TIME *);
short VerifyPlaceStruct(struct PLACE *);

/* ====================================================== */
/* = calend.c PROTOTYPES                                = */
/* ====================================================== */
short Calend(struct TIME *, struct PLACE *);

/* ====================================================== */
/* = jupiter.c PROTOTYPES                               = */
/* ====================================================== */
short JSat(struct TIME *, struct PLACE *, struct PARAM *);
short DrawJSat(float, float, float, float, short, char *);
void  DrawGRS(float, float);
short JTransit(float, struct PARAM *);
short DrawJSatShadow(float, float, float, float);

/* ====================================================== */
/* = mars.c PROTOTYPES                                  = */
/* ====================================================== */
short MarsView(struct TIME *, struct PLACE *, struct PARAM *);

/* ====================================================== */
/* = saturne.c PROTOTYPES                               = */
/* ====================================================== */
short RingSat(struct TIME *, struct PLACE *, struct PARAM *);